"""
LeMMA - the Lazy man's Editor for MMA
Note that this is NOT really an MMA editor, but rather a simple front-end to MMA

This is the main application module

Software is provided AS IS. Feel free to use AT YOUR OWN RISK!

Version 0.5. Copyright (c) Gek S. Low. 8 Dec 2007
"""


# app.py

from Tkinter import *
import tkMessageBox
import tkFileDialog
import re
import cPickle

import settings

version = "0.5 (8 Dec 2007)"
MMA_filetypes = (("MMA files", "*.mma"), ("All files","*.*"))

def blah():
	tkMessageBox.showinfo("Under construction", "This feature is not yet implemented")

class Application(Frame):
	def __init__(self, master=None):
		Frame.__init__(self, master)
		self.grid()
		self.createWidgets()
		settings.readGrooves()
		settings.readSettings()
		
	def createWidgets(self):
		# Main toolbar
		self.BtnFrame = Frame(self)
		self.BtnFrame.grid(row=0, column=0, sticky=NW)
		
		self.newBtn = Button(self.BtnFrame, text="New", command=self.newFile)
		self.newBtn.grid(row=0, column=0, sticky=NW)
		self.loadMMABtn = Button(self.BtnFrame, text="Load MMA", command=self.loadMMA)
		self.loadMMABtn.grid(row=0, column=1, sticky=NW)
		self.saveMMABtn = Button(self.BtnFrame, text="Save MMA", command=self.saveAsMMA)
		self.saveMMABtn.grid(row=0, column=3, sticky=NW)
		self.playBtn = Button(self.BtnFrame, text="Play", command=self.playFile)
		self.playBtn.grid(row=0, column=4, sticky=NW)
		self.settingsBtn = Button(self.BtnFrame, text="Settings", command=self.settings)
		self.settingsBtn.grid(row=0, column=5, sticky=NW)
		self.aboutBtn = Button(self.BtnFrame, text="About", command=self.about)
		self.aboutBtn.grid(row=0, column=6, sticky=NW)
		self.helpBtn = Button(self.BtnFrame, text="Help", command=self.help)
		self.helpBtn.grid(row=0, column=7, sticky=NW)
		self.quitBtn = Button(self.BtnFrame, text="Quit", command=self.quit)
		self.quitBtn.grid(row=0, column=8, sticky=NW)
				
		# Time Signature, Key Signature, Tempo
		self.infoFrame = Frame(self)
		self.infoFrame.grid(row=1, column=0, sticky=NW, padx=5, pady=5)
		
		Label(self.infoFrame, text="Time Sig ").grid(row=0, column=0, sticky=NW)
		self.timeSigEntry = Entry(self.infoFrame, width=4)
		self.timeSigEntry.insert(1,"4/4")
		self.timeSigEntry.grid(row=0, column=1, sticky=NW)
		
		Label(self.infoFrame, text="   Key ").grid(row=0, column=2, sticky=NW)
		self.keyEntry = Entry(self.infoFrame, width=6)
		self.keyEntry.insert(1,"C")
		self.keyEntry.grid(row=0, column=3, sticky=NW)
		
		Label(self.infoFrame, text="   Tempo ").grid(row=0, column=4, sticky=NW)
		self.tempoEntry = Entry(self.infoFrame, width=4)
		self.tempoEntry.insert(1,"60")
		self.tempoEntry.grid(row=0, column=5, sticky=NW)
		
		self.SwingMode = 1
		self.swingCheckBtn = Checkbutton(self.infoFrame, text="Swing Mode", command=self.toggleSwingMode)
		self.swingCheckBtn.grid(row=0, column=6, sticky=NW)
		self.swingCheckBtn.select()
		
		# This is where the chords are entered
		self.mainFrame = Frame(self)
		self.mainFrame.grid(row=2, column=0, sticky=NW, padx=5, pady=5)
		
		self.measures = []
		self.grooves = []
		# 40 measures arranged in groups of 4 - fixed for now. change y value if you need more...
		for y in range(10):
			for x in range(4):
				self.measures += [Measure(self.mainFrame)]
				def setGroove(index=x+y*4):
					d = settings.GroovesDialog(self)
					if d.selectedGroove != "":
						self.grooves[index].configure(text=d.selectedGroove, foreground="black")
					else:
						self.grooves[index].configure(text="(Groove)", foreground="gray")
				self.grooves += [Button(self.mainFrame, text="(Groove)", foreground="gray", justify=LEFT, relief=FLAT, takefocus=0, command=setGroove)]
				
				Label(self.mainFrame, text=str(x+1+y*4), justify=RIGHT, takefocus=0).grid(row=y*2+1, column=x*2, sticky=SE)
				self.measures[x+y*4].grid(row=y*2+1, column=x*2+1, sticky=NW)
				self.grooves[x+y*4].grid(row=y*2, column=x*2+1, sticky=NW)
		self.measures[0].contents.focus()
		
		self.outputText = Text(self, width=70, height=5, bg="lightgray", wrap=WORD, relief=GROOVE)
		self.outputText.grid(row=3, column=0, sticky=N)
		
		settings.outputWindow = self.outputText
	
	def toggleSwingMode(self):
		self.SwingMode = not self.SwingMode
	
	def newFile(self):
		if tkMessageBox.askyesno("Warning", "All contents will be lost! Continue?"):
			for measure in self.measures:
				measure.contents.delete(0, END)
			for groove in self.grooves:
				groove.configure(text="(Groove)", foreground="gray")
				
	def loadMMA(self):
		global MMA_filetypes
		
		# reset
		for measure in self.measures:
			measure.contents.delete(0, END)
		for groove in self.grooves:
			groove.configure(text="(Groove)", foreground="gray")
			
		self.currentFile = tkFileDialog.askopenfilename(filetypes = MMA_filetypes)
		
		if self.currentFile == "":
			return
		
		f = open(self.currentFile, "r")
		i = 0 # measure number
		
		measure = 0
		repeatending_count = 1
		clearcontents = 1
		
		# read each line
		for line in f:
			line = line.strip()
			
			if re.compile(r'KeySig\s*').match(line):
				line = re.compile(r'KeySig\s*').sub("", line)
				self.keyEntry.delete(0, END)
				self.keyEntry.insert(0, line)
				continue
			
			if re.compile(r'TimeSig\s*').match(line):
				line = re.compile(r'TimeSig\s*').sub("", line)
				line = line.replace(" " , "/")
				self.timeSigEntry.delete(0, END)
				self.timeSigEntry.insert(0, line)
				continue
			
			if re.compile(r'Tempo\s*').match(line):
				line = re.compile(r'Tempo\s*').sub("", line)
				self.tempoEntry.delete(0, END)
				self.tempoEntry.insert(0, line)
				continue
			
			if re.compile(r'SwingMode\s*').match(line):
				line = re.compile(r'SwingMode\s*').sub("", line)
				if line == "On":
					self.swingCheckBtn.select()
				else:
					self.swingCheckBtn.deselect()
				continue
				
			if re.compile(r'Groove\s*').match(line):
				line = re.compile(r'Groove\s*').sub("", line)
				self.grooves[measure].configure(text=line, foreground="black")
				continue
			
			if clearcontents == 1:
				contents = ""
			
			# is this a repeat, repeatending?
			if line == "Repeat":
				contents = "|: "
				repeatending_count = 1
				clearcontents = 0
			if line == "RepeatEnding":
				contents = "[" + str(repeatending_count) + " "
				repeatending_count += 1
				clearcontents = 0
				
			# form the measure contents field
			if re.compile(r'\d+').match(line):
				line = re.compile(r'^\d+\s*').sub("", line)
				contents = contents + line
				clearcontents = 1
			
			# is this a repeat end? then need to retrieve and add to previous bar
			if line == "RepeatEnd":
				measure = measure - 1
				contents = self.measures[measure].contents.get()
				self.measures[measure].contents.delete(0, END)
				contents += " :|"
				clearcontents = 1
			
			# insert into measure
			if clearcontents == 1 and contents != "":
				self.measures[measure].contents.insert(0, contents)
				measure = measure + 1
				if measure > len(self.measures):
					break
			
			if line == "cut -1":
				break
				
		f.close()
		settings.printOutput("Loaded " + self.currentFile)
	
	def playFile(self):
		self.saveMMA("_temp_.mma")
		settings.playMMA()
		
	def saveAsMMA(self):
		self.currentFile = tkFileDialog.asksaveasfilename(filetypes = MMA_filetypes)
		if self.currentFile != "" and self.currentFile != ():
			self.saveMMA(self.currentFile)
		settings.printOutput("Saved " + self.currentFile)
		
	def saveMMA(self, filename):
		f = open(filename, "w")
		i = 0	# measure number
		print >>f, "// MMA file generated by LeMMA version " + version + "\n"
		print >>f, "KeySig " + self.keyEntry.get()
		print >>f, "TimeSig " + self.timeSigEntry.get().replace("/", " ")
		print >>f, "Tempo " + self.tempoEntry.get() + "\n"
		if self.SwingMode == 1:
			print >>f, "SwingMode On\n"
		else:
			print >>f, "SwingMode Off\n"
			
		#swingmode?
		reRepeatEnding = re.compile(r'\[\d+')
		for measure in self.measures:
			groove = self.grooves[i]["text"]
			
			# set default groove (Folk) if first measure is empty
			if i == 0 and groove == "(Groove)":
				groove = "Folk"
		
			contents = measure.contents.get()
			
			isRepeat = 0
			isRepeatEnd = 0
			isRepeatEnding = 0
			
			if contents.find("|:") != -1:
				isRepeat = 1
				contents = contents.replace("|:","")
				
			if contents.find(":|") != -1:
				isRepeatEnd = 1
				contents = contents.replace(":|","")
				
			if reRepeatEnding.match(contents):
				isRepeatEnding = 1
				contents = reRepeatEnding.sub("", contents) # first repeat
			
			contents = contents.strip(' ')
			if contents != "":
				# print the groove first, if any
				if groove != "(Groove)":
					print >>f, "Groove " + groove
				# now for the measure
				if isRepeat == 1:
					print >>f, "Repeat"
				if isRepeatEnding == 1:
					print >>f, "RepeatEnding"
				
				print >>f, str(i+1) + " " + contents
				
				if isRepeatEnd == 1:
					print >>f, "RepeatEnd"
			else:
				break	# stops at first empty measure, ignoring any other non-empty measures after it
			i = i+1
			
		print >>f, "cut -1"
		f.close()
		
	def about(self):
		tkMessageBox.showinfo("About LeMMA",
			"LeMMA - the Lazyman's editor for MMA\n\n"
			"Version " + version + "\n\n"
			"Copyright (c) Gek S. Low")
	
	def help(self):
		d = settings.HelpDialog(self)

	def settings(self):
		d = settings.SettingsDialog(self)

class Measure(Frame):
	def __init__(self, master=None):
		Frame.__init__(self, master, borderwidth=1, bg="white")
		self.grid()
		self.createWidgets()
	
	def createWidgets(self):
		self.contents = Entry(self, width=12, borderwidth=0, highlightcolor="black", highlightthickness=2, highlightbackground="white", relief=FLAT, font=("Helvetica", "12"), justify=LEFT)
		self.contents.grid(row=0,column=0)
			
		def updatewidth(event, self=self):
			return self.__updatewidth(event)
		self.contents.bind("<FocusOut>", updatewidth)
			
	def __updatewidth(self, event):
		w = len(str(self.contents.get()))
		if w >= 12:
			self.contents.configure(width=w+2)
		else:
			self.contents.configure(width=12)
		self.contents.index(1)
