"""
LeMMA - the Lazy man's Editor for MMA
Note that this is NOT really an MMA editor, but rather a simple front-end to MMA

This is the module that deals with (mostly) configuration stuff

Software is provided AS IS. Feel free to use AT YOUR OWN RISK!

Version 0.5. Copyright (c) Gek S. Low. 8 Dec 2007
"""

# Settings
from Tkinter import *
import os
import re
import tkFileDialog
import tkSimpleDialog
import subprocess
import cPickle
import commands
import platform

if platform.system() == "Linux":
	isLinux = 1
else:
	isLinux = 0

if isLinux == 0:
	mmaPath = "c:\MMA\mma.py"
	midiPlayer = ""
	pythonPath = "c:\\python24\\python"
else:
	mmaPath = "/usr/local/bin/mma"
	midiPlayer = "/usr/bin/audacious"
	pythonPath = "/usr/bin/python"

groovedict = {}
currentgroovelib = ""

outputWindow = Text()

def printOutput(text):
	outputWindow.delete("1.0", END)
	outputWindow.insert("1.0", text)
	outputWindow.update()
	
def readGrooves():
	global groovedict
	try:
		f = open("grooves.dat", "r")
		p = cPickle.Unpickler(f)
		groovedict = p.load()
		f.close()
	except IOError:
		printOutput("Error opening grooves.dat")

def readSettings():
	global mmaPath
	global midiPlayer
	global pythonPath
	
	# load settings (if file exists)
	try:
		f = open("settings.dat", "r")
		p = cPickle.Unpickler(f)
		pythonPath = p.load()
		mmaPath = p.load()
		midiPlayer = p.load()
		f.close()
	except IOError:
		printOutput("Error opening settings.dat")

def playMMA():
	global mmaPath
	global midiPlayer
	# run MMA - must run from mma.py path
	currentdir = os.getcwd()
	mmaDir = re.compile(r'.*[\\/]').match(mmaPath).group()
	os.chdir(mmaDir)
	
	cmd = pythonPath + " " + mmaPath + " " + currentdir + "/_temp_.mma"
	print cmd
	pipe = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
	output = pipe.read()
	status = pipe.close()

	os.chdir(currentdir)
	
	#print output
	printOutput(output)
	
	# play Midi
	if isLinux == 0:
		# On windows, can't use os.popen as that will wait for process to finish, then I'll have to close the midi player all the time
		# On windows, need shell=False to avoid a shell window
		cmd = "\"" + midiPlayer + "\" " + currentdir + "/_temp_.mid"
		pipe = subprocess.Popen(cmd, shell=False, stdout=None).stdout
	else:
		# On Linux, need shell=True for this to work, but note that there is no way to control/stop timidity.
		cmd = midiPlayer + " " + currentdir + "/_temp_.mid"
		pipe = subprocess.Popen(cmd, shell=True, stdout=None).stdout
		#pipe = os.popen(cmd, 'r')

class HelpDialog(tkSimpleDialog.Dialog):
	def body(self, master):
		# read help file
		try:
			f = open("help.txt", "r")
			help = f.read()
			f.close()
		except:
			printOutput("Can't open help.txt")
			return
		
		self.helpText = Text(master, width=50, height=20, bg="lightgray", wrap=WORD)
		self.helpText.insert("1.0", help)
		self.helpText.grid(row=0, column=0, sticky=N)
		self.helpScroll = Scrollbar(master, orient=VERTICAL, command=self.helpText.yview)
		self.helpScroll.grid(row=0, column=1, stick=NS)
		
		self.helpText["yscrollcommand"] = self.helpScroll.set
		
		return self.helpText
	
	def apply(self):
		return
	
class SettingsDialog(tkSimpleDialog.Dialog):
	def body(self, master):
		global mmaPath
		global midiPlayer
		
		Label(master, text="Python path ").grid(row=0, column=0, sticky=NW)
		self.pythonPathEntry = Entry(master, width=20)
		self.pythonPathEntry.insert(1, pythonPath)
		self.pythonPathEntry.grid(row=0, column=1, sticky=NW)
		self.pythonPathBrowseBtn = Button(master, text="Browse", command=self.pythonPathBrowse)
		self.pythonPathBrowseBtn.grid(row=0, column=2, sticky=NW)
		
		Label(master, text="MMA path ").grid(row=1, column=0, sticky=NW)
		self.mmaPathEntry = Entry(master, width=20)
		self.mmaPathEntry.insert(1, mmaPath)
		self.mmaPathEntry.grid(row=1, column=1, sticky=NW)
		self.mmaPathBrowseBtn = Button(master, text="Browse", command=self.mmaPathBrowse)
		self.mmaPathBrowseBtn.grid(row=1, column=2, sticky=NW)
	
		Label(master, text="Midi player ").grid(row=2, column=0, sticky=NW)
		self.midiPlayerEntry = Entry(master, width=20)
		self.midiPlayerEntry.insert(1, midiPlayer)
		self.midiPlayerEntry.grid(row=2, column=1, sticky=NW)
		self.midiPlayerBrowseBtn = Button(master, text="Browse", command=self.midiPlayerBrowse)
		self.midiPlayerBrowseBtn.grid(row=2, column=2, sticky=NW)
		
		self.mmaPathRefreshBtn = Button(master, text="Refresh grooves library", command=self.refreshGrooves)
		self.mmaPathRefreshBtn.grid(row=3, column=1, sticky=NW)

		return self.mmaPathEntry	# initial focus
	
	def mmaPathBrowse(self):
		currentFile = tkFileDialog.askopenfilename()
		if currentFile != "" and currentFile != ():
			self.mmaPathEntry.delete(0, END)
			self.mmaPathEntry.insert(1, currentFile)

	def pythonPathBrowse(self):
		currentFile = tkFileDialog.askopenfilename()
		if currentFile != "" and currentFile != ():
			self.pythonPathEntry.delete(0, END)
			self.pythonPathEntry.insert(1, currentFile)
	
	def midiPlayerBrowse(self):
		currentFile = tkFileDialog.askopenfilename()
		if currentFile != "" and currentFile != ():
			# check whether there's a space in the path, if so wrap it in quotes
			if currentFile.find(" "):
				currentFile = "\"" + currentFile + "\""
			self.midiPlayerEntry.delete(0, END)
			self.midiPlayerEntry.insert(1, currentFile)
		
	def apply(self):
		global mmaPath
		global midiPlayer
		global pythonPath
		
		pythonPath = self.pythonPathEntry.get()
		mmaPath = self.mmaPathEntry.get()
		midiPlayer = self.midiPlayerEntry.get()

		# strip trailing spaces
		pythonPath = pythonPath.strip(' ')
		mmaPath = mmaPath.strip(' ')
		midiPlayer = midiPlayer.strip(' ')
		
		# save the settings into settings file
		f = open("settings.dat", "w")
		p = cPickle.Pickler(f)
		p.dump(pythonPath)
		p.dump(mmaPath)
		p.dump(midiPlayer)
		f.close()

	def refreshGrooves(self):
		global mmaPath
		# get all groove files, note this takes a while to complete
		
		mmaDir = re.compile(r'.*[\\/]').match(mmaPath).group()
		# try different locations for the libraries
		# try the lib directory from mma path first
		if os.path.exists(mmaDir + "lib"):
			libDir = mmaDir + "lib"
		elif os.path.exists("/usr/share/mma/lib"):
			libDir = "/usr/share/mma/lib"
		elif os.path.exists("/usr/local/share/mma/lib"):
			libDir = "/usr/local/share/mma/lib"
		else:
			libDir = "";
			printOutput("Couldn't find the Grooves directory")
			return

		tree = os.walk(libDir)
		currentdir = os.getcwd()
		os.chdir(mmaDir)
		
		global groovedict
		global outputWindow
					
		groovedict = {}
		groovecount = 0
		p1 = re.compile(r'\\filehead\{(.*?)\}\{(.*?)\}')
		p2 = re.compile(r'\\instable\{(.*?)\}\{(.*?)\}')
		
		# check MMA version
		mmaversion = 1.0
		try:
			cmd = pythonPath + " " + mmaPath + " -v"
			pipe = os.popen(cmd, 'r')
			mmaversion = float(pipe.read())
			status = pipe.close()
		except:
			printOutput("Oops, an error was encountered")
						
		for directory in tree:
			for file in directory[2]:
				if file.endswith("mma"):
					filename = directory[0] + "/" + file
					# run MMA command
					try:
						# NOTE: In Windows, I need a shell window for this to work!
						if mmaversion > 1.0:
							cmd = pythonPath + " " + mmaPath + " -Dxl " + filename
						else:
							cmd = pythonPath + " " + mmaPath + " -Dx " + filename
						pipe = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
						#pipe = os.popen(cmd, 'r')
						grooveinfo = pipe.read()
						status = pipe.close()
						
						#print grooveinfo
					except:
						printOutput("Oops, an error was encountered")
						
					m1 = p1.search(grooveinfo)
					groovelibrary = m1.group(1)
					groovedict[groovelibrary] = [m1.group(2)]
					
					m2 = p2.findall(grooveinfo)
					grooves = []
					
					printOutput("Reading grooves...")
					
					for groove in m2:	
						groovename = groove[0]
						groovedesc = groove[1]
						
						groovename = groovename.replace("\\\\","\\")
						groovename = groovename.replace("\\&","&")
						groovename = groovename.replace("``","`")
						groovename = groovename.replace("\"\"","\"")
						groovename = groovename.replace("\'\'","\'")
						
						groovedesc = groovedesc.replace("\\\\","\\")
						groovedesc = groovedesc.replace("\\&","&")
						groovedesc = groovedesc.replace("``","`")
						groovedesc = groovedesc.replace("\"\"","\"")
						groovedesc = groovedesc.replace("\'\'","\'")
						
						#print groovename
						
						outputWindow.insert("1.end", " "+groovename)
						groovecount += 1
						
						grooves += [[groovename, groovedesc]]
					groovedict[groovelibrary] += [grooves]
					
					outputWindow.update()
		os.chdir(currentdir)
		
		printOutput("Read "+str(groovecount)+" grooves")
		
		#print groovedict
		#write groovedict to grooves file
		f = open("grooves.dat", "w")
		p = cPickle.Pickler(f)
		p.dump(groovedict)
		f.close()


class GroovesDialog(tkSimpleDialog.Dialog):
	selectedGroove = ""
	
	def body(self, master):
		global groovedict
		global currentgroovelib
		keylist = groovedict.keys()
		keylist.sort()
		if currentgroovelib == "":
			currentgroovelib = keylist[0]
		
		self.mb = Menubutton(master, text=currentgroovelib, direction=RIGHT)
		self.mb.grid(row=0, column=0, sticky=NW)
	
		# the groove library names go into a menu
		self.mb.menu = Menu(self.mb, tearoff=0)
		self.mb["menu"] = self.mb.menu

		# define several cascade menus, we'll sort by alphabet
		menus = {}
		menuitems = {}
		for ch in "#ABCDEFGHIJKLMNOPQRSTUVWXYZ":
			menus[ch] = Menu(self.mb, tearoff=0)
			menuitems[ch] = 0
			self.mb.menu.add_cascade(menu=menus[ch], label=ch)
		
		for key in keylist:
			def updateListBox(key=key):
				self.__updateListBox(key)
			if key[0] in "0123456789":
				menus["#"].add_command(label=key, command=updateListBox)
				menuitems["#"] += 1
			elif key[0].upper() in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
				menus[key[0].upper()].add_command(label=key, command=updateListBox)
				menuitems[key[0].upper()] += 1
			else:
				print key
			#self.mb.menu.add_command(label=key, command=updateListBox)
		# disable the cascade menus which are empty
		index = 0
		for ch in "#ABCDEFGHIJKLMNOPQRSTUVWXYZ":
			if menuitems[ch] == 0:
				self.mb.menu.entryconfigure(index, state=DISABLED)
			index += 1

		# which refreshes a listbox (with description)
		self.label = Label(master, text=groovedict[currentgroovelib][0], width=50, wraplength=250)
		self.label.grid(row=1, column=0, sticky=NW)
		
		self.lb = Listbox(master, width=50)
		grooves = groovedict[currentgroovelib][1]
		grooves.sort()
		for groove in grooves:
			self.lb.insert(1, groove[0] + ": " + groove[1])
		self.lb.grid(row=2, column=0, stick=NW)
		
		return self.mb	# initial focus
		
	def __updateListBox(self, key):
		global groovedict
		global currentgroovelib
		if key != currentgroovelib:
			self.lb.delete(0, END)
			grooves = groovedict[key][1]
			grooves.sort()
			for groove in grooves:
				self.lb.insert(1, groove[0] + ": " + groove[1])
			currentgroovelib = key
			self.mb.configure(text=key)
			self.label.configure(text=groovedict[key][0])

	def apply(self):
		if self.lb.curselection() != ():
			self.selectedGroove = self.lb.get(self.lb.curselection())
			self.selectedGroove = re.compile(r':.*').sub("", self.selectedGroove)
		
